@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_asf.c,v 1.167.2.2 2005/04/20 17:21:02 mroi Exp $
+ * $Id: demux_asf.c,v 1.172 2005/07/18 01:28:21 dsalt Exp $
  *
  * demultiplexer for asf streams
  *
@@ -97,6 +97,10 @@ typedef struct {
 
   int                 payload_size;
 
+  /* palette handling */
+  int               palette_count;
+  palette_entry_t   palette[256];
+  
 } asf_stream_t;
 
 typedef struct demux_asf_s {
@@ -360,10 +364,24 @@ static void asf_send_video_header (demux_asf_t *this, int stream) {
   buf->decoder_info[0] = 0;
   buf->size = asf_stream->bih_size;
   memcpy (buf->content, bih, buf->size);
-  buf->type = this->streams[stream].buf_type ;
+  buf->type = this->streams[stream].buf_type;
 
   this->video_fifo->put (this->video_fifo, buf);
 
+
+  /* send off the palette, if there is one */
+  if (asf_stream->palette_count) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+	    "demux_asf: stream %d, palette : %d entries\n", stream, asf_stream->palette_count);
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+    buf->decoder_flags = BUF_FLAG_SPECIAL|BUF_FLAG_HEADER;
+    buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
+    buf->decoder_info[2] = asf_stream->palette_count;
+    buf->decoder_info_ptr[2] = &asf_stream->palette;
+    buf->size = 0;
+    buf->type = this->streams[stream].buf_type;
+    this->video_fifo->put (this->video_fifo, buf);
+  }
 }
 
 static int asf_read_header (demux_asf_t *this) {
@@ -534,6 +552,7 @@ static int asf_read_header (demux_asf_t *this) {
             uint16_t i;
             uint32_t width;
             uint32_t height;
+	    int j;
 
             width = get_le32(this); /* width */
             height = get_le32(this); /* height */
@@ -565,6 +584,31 @@ static int asf_read_header (demux_asf_t *this) {
               this->streams[this->num_streams].frag_offset  = 0;
               this->streams[this->num_streams].defrag       = 0;
 
+	      /* load the palette, if there is one */
+	      asf_stream->palette_count = asf_stream->bih->biClrUsed;
+
+	      lprintf ("palette_count: %d\n", asf_stream->palette_count);
+	      if (asf_stream->palette_count > 256) {
+		lprintf ("number of colors exceeded 256 (%d)", asf_stream->palette_count);
+		asf_stream->palette_count = 256;
+	      }
+	      if ((asf_stream->bih_size - sizeof(xine_bmiheader)) >= (asf_stream->palette_count * 4)) {
+		/* load the palette from the end of the strf chunk */
+		for (j = 0; j < asf_stream->palette_count; j++) {
+		  asf_stream->palette[j].b = *((uint8_t *)asf_stream->bih + sizeof(xine_bmiheader) + j * 4 + 0);
+		  asf_stream->palette[j].g = *((uint8_t *)asf_stream->bih + sizeof(xine_bmiheader) + j * 4 + 1);
+		  asf_stream->palette[j].r = *((uint8_t *)asf_stream->bih + sizeof(xine_bmiheader) + j * 4 + 2);
+		}
+	      } else {
+		/* generate a greyscale palette */
+		asf_stream->palette_count = 256;
+		for (j = 0; j < asf_stream->palette_count; j++) {
+		  asf_stream->palette[j].r = j;
+		  asf_stream->palette[j].g = j;
+		  asf_stream->palette[j].b = j;
+		}
+	      }
+
             } else
               xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
 		       "demux_asf: invalid bih_size received (%d), v_stream ignored.\n", i );
@@ -1304,7 +1348,7 @@ static int asf_parse_packet_compressed_payload(demux_asf_t *this,
 
   *timestamp = frag_offset;
   if (*timestamp)
-  	*timestamp -= this->preroll;
+    *timestamp -= this->preroll;
 
   frag_offset = 0;
   get_byte (this); s_hdr_size += 1;
@@ -1492,8 +1536,6 @@ static int demux_asf_parse_http_references( demux_asf_t *this) {
   int             buf_used = 0;
   int             len;
   char           *href = NULL;
-  xine_mrl_reference_data_t *data;
-  xine_event_t    uevent;
   char           *mrl;
   int             free_href = 0;
 
@@ -1544,15 +1586,7 @@ static int demux_asf_parse_http_references( demux_asf_t *this) {
     }
     
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_asf: http ref: %s\n", href);
-    uevent.type = XINE_EVENT_MRL_REFERENCE;
-    uevent.stream = this->stream;
-    uevent.data_length = strlen(href) + sizeof(xine_mrl_reference_data_t);
-    data = malloc(uevent.data_length);
-    uevent.data = data;
-    strcpy(data->mrl, href);
-    data->alternative = 0;
-    xine_event_send(this->stream, &uevent);
-    free(data);
+    _x_demux_send_mrl_reference (this->stream, 0, href, NULL, 0, 0);
     
     if (free_href)
       free(href);
@@ -1574,8 +1608,6 @@ static int demux_asf_parse_asf_references( demux_asf_t *this) {
   int             buf_size = 0;
   int             buf_used = 0;
   int             len;
-  xine_mrl_reference_data_t *data;
-  xine_event_t    uevent;
   int             i;
 
   /* read file to memory.
@@ -1610,15 +1642,7 @@ static int demux_asf_parse_asf_references( demux_asf_t *this) {
     }
 
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_asf: asf ref: %s\n", ptr);
-    uevent.type = XINE_EVENT_MRL_REFERENCE;
-    uevent.stream = this->stream;
-    uevent.data_length = strlen(ptr) + sizeof(xine_mrl_reference_data_t);
-    data = malloc(uevent.data_length);
-    uevent.data = data;
-    strcpy(data->mrl, ptr);
-    data->alternative = 0;
-    xine_event_send(this->stream, &uevent);
-    free(data);
+    _x_demux_send_mrl_reference (this->stream, 0, ptr, NULL, 0, 0);
   }
 
   free (buf);
@@ -1627,6 +1651,29 @@ static int demux_asf_parse_asf_references( demux_asf_t *this) {
   return this->status;
 }
 
+
+/* .asx playlist parser helper functions */
+static uint32_t asx_get_time_value (const xml_node_t *node)
+{
+  const char *value = xml_parser_get_property (node, "VALUE");
+
+  if (value)
+  {
+    int hours, minutes;
+    double seconds;
+
+    if (sscanf (value, "%d:%d:%lf", &hours, &minutes, &seconds) == 3)
+      return hours * 3600000 + minutes * 60000 + seconds * 1000;
+
+    if (sscanf (value, "%d:%lf", &minutes, &seconds) == 3)
+      return minutes * 60000 + seconds * 1000;
+
+    /* FIXME: single element is minutes or seconds? */
+  }
+
+  return 0; /* value not found */
+}
+
 /*
  * parse .asx playlist files
  */
@@ -1636,10 +1683,7 @@ static int demux_asf_parse_asx_references( demux_asf_t *this) {
   int             buf_size = 0;
   int             buf_used = 0;
   int             len;
-  xine_mrl_reference_data_t *data;
-  xine_event_t    uevent;
   xml_node_t     *xml_tree, *asx_entry, *asx_ref;
-  xml_property_t *asx_prop;
   int             result;
 
 
@@ -1667,75 +1711,108 @@ static int demux_asf_parse_asx_references( demux_asf_t *this) {
     goto failure;
 
   if(!strcasecmp(xml_tree->name, "ASX")) {
+    /* Attributes: VERSION, PREVIEWMODE, BANNERBAR
+     * Child elements: ABSTRACT, AUTHOR, BASE, COPYRIGHT, DURATION, ENTRY,
+                       ENTRYREF, MOREINFO, PARAM, REPEAT, TITLE
+     */
 
-    asx_prop = xml_tree->props;
+    const char *version = xml_parser_get_property (xml_tree, "VERSION");
 
-    while((asx_prop) && (strcasecmp(asx_prop->name, "VERSION")))
-      asx_prop = asx_prop->next;
-
-    if(asx_prop) {
+    if (version) {
       int  version_major, version_minor = 0;
 
-      if((((sscanf(asx_prop->value, "%d.%d", &version_major, &version_minor)) == 2) ||
-          ((sscanf(asx_prop->value, "%d", &version_major)) == 1)) &&
-         ((version_major == 3) && (version_minor == 0))) {
+      if((sscanf (version, "%d.%d", &version_major, &version_minor) == 2 ||
+          sscanf (version, "%d", &version_major) == 1) &&
+         (version_major == 3 && version_minor == 0))
+      {
+        const char *base_href = NULL;
 
-        asx_entry = xml_tree->child;
-        while(asx_entry) {
-          if((!strcasecmp(asx_entry->name, "ENTRY")) ||
-             (!strcasecmp(asx_entry->name, "ENTRYREF"))) {
-            char *href   = NULL;
+        for (asx_entry = xml_tree->child; asx_entry; asx_entry = asx_entry->next)
+        {
+          const char *ref_base_href = base_href;
 
-            asx_ref = asx_entry->child;
-            if (!asx_ref && !strcasecmp(asx_entry->name, "ENTRYREF")) {
-              for(asx_prop = asx_entry->props; asx_prop; asx_prop = asx_prop->next)
+          if (!strcasecmp (asx_entry->name, "ENTRY"))
+          {
+            /* Attributes: CLIENTSKIP, SKIPIFREF
+             * Child elements: ABSTRACT, AUTHOR, BASE, COPYRIGHT, DURATION,
+                               ENDMARKER, MOREINFO, PARAM, REF, STARTMARKER,
+                               STARTTIME, TITLE
+             */
+            const char *href = NULL;
+            const char *title = NULL;
+            uint32_t start_time = -1;
+            uint32_t duration = -1;
 
-                if(!strcasecmp(asx_prop->name, "HREF")) {
+            for (asx_ref = asx_entry->child; asx_ref; asx_ref = asx_ref->next)
+            {
+              if (!strcasecmp(asx_ref->name, "REF"))
+              {
+                xml_node_t *asx_sub;
+                /* Attributes: HREF
+                 * Child elements: DURATION, ENDMARKER, STARTMARKER, STARTTIME
+                 */
 
-                    href = asx_prop->value;
+                /* FIXME: multiple REFs => alternative streams
+                 * (and per-ref start times and durations?).
+                 * Just the one title, though.
+                 */
+                href = xml_parser_get_property (asx_ref, "HREF");
 
-                    if(href)
-                      break;
-                }
-            }
-
-            while(asx_ref) {
-
-              if(!strcasecmp(asx_ref->name, "REF")) {
-
-                for(asx_prop = asx_ref->props; asx_prop; asx_prop = asx_prop->next) {
-
-                  if(!strcasecmp(asx_prop->name, "HREF")) {
-
-                    if(!href)
-                      href = asx_prop->value;
-                  }
-                  if(href)
-                    break;
+                for (asx_sub = asx_ref->child; asx_sub; asx_sub = asx_sub->next)
+                {
+                  if (!strcasecmp (asx_sub->name, "STARTTIME"))
+                    start_time = asx_get_time_value (asx_sub);
+                  else if (!strcasecmp (asx_sub->name, "DURATION"))
+                    duration = asx_get_time_value (asx_sub);
                 }
               }
-              asx_ref = asx_ref->next;
+
+              else if (!strcasecmp (asx_ref->name, "TITLE"))
+              {
+                if (!title)
+                  title = asx_ref->data;
+              }
+
+              else if (!strcasecmp (asx_ref->name, "STARTTIME"))
+              {
+                if (start_time < 0) 
+                  start_time = asx_get_time_value (asx_ref);
+              }
+
+              else if (!strcasecmp (asx_ref->name, "DURATION"))
+              {
+                if (duration < 0) 
+                  duration = asx_get_time_value (asx_ref);
+              }
+
+              else if (!strcasecmp (asx_ref->name, "BASE"))
+                /* Attributes: HREF */
+                ref_base_href = xml_parser_get_property (asx_entry, "HREF");
             }
 
-            if(href && strlen(href)) {
-              uevent.type = XINE_EVENT_MRL_REFERENCE;
-              uevent.stream = this->stream;
-              uevent.data_length = strlen(href)+sizeof(xine_mrl_reference_data_t);
-              data = malloc(uevent.data_length);
-              uevent.data = data;
-              strcpy(data->mrl, href);
-              data->alternative = 0;
-              xine_event_send(this->stream, &uevent);
-              free(data);
-            }
-            href = NULL;
+            /* FIXME: prepend ref_base_href to href */
+            if (href && *href)
+              _x_demux_send_mrl_reference (this->stream, 0, href, title,
+                                           start_time < 0 ? 0 : start_time,
+                                           duration < 0 ? -1 : duration);
           }
-          asx_entry = asx_entry->next;
+
+          else if (!strcasecmp (asx_entry->name, "ENTRYREF"))
+          {
+            /* Attributes: HREF, CLIENTBIND */
+            const char *href = xml_parser_get_property (asx_entry, "HREF");
+            if (href && *href)
+              _x_demux_send_mrl_reference (this->stream, 0, href, NULL, 0, -1);
+          }
+
+          else if (!strcasecmp (asx_entry->name, "BASE"))
+            /* Attributes: HREF */
+            base_href = xml_parser_get_property (asx_entry, "HREF");
         }
       }
       else
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-		_("demux_asf: Wrong ASX version: %s\n"), asx_prop->value);
+		_("demux_asf: Wrong ASX version: %s\n"), version);
       
     }
     else
