@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -20,7 +20,7 @@
  * Compact Disc Digital Audio (CDDA) Input Plugin 
  *   by Mike Melanson (melanson@pcisys.net)
  *
- * $Id: input_cdda.c,v 1.69.2.2 2005/07/17 21:49:58 miguelfreitas Exp $
+ * $Id: input_cdda.c,v 1.72 2005/05/28 09:26:59 jstembridge Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,12 +32,7 @@
 #include <unistd.h>
 
 #include <sys/types.h>
-#ifdef SYS_PARAM_H
-#include <sys/param.h>
-#endif
-#ifdef HAVE_DIRENT_H
 #include <dirent.h>
-#endif
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -689,26 +684,18 @@ static int read_cdrom_frames(cdda_input_plugin_t *this_gen, int frame, int num_f
   unsigned char *data) {
 
   int fd = this_gen->fd;
-#if  __FreeBSD_version < 501106
   struct ioc_read_audio cdda;
-#endif
 
   while( num_frames ) {
-#if  __FreeBSD_version < 501106
     cdda.address_format = CD_MSF_FORMAT;
     cdda.address.msf.minute = frame / CD_SECONDS_PER_MINUTE / CD_FRAMES_PER_SECOND;
     cdda.address.msf.second = (frame / CD_FRAMES_PER_SECOND) % CD_SECONDS_PER_MINUTE;
     cdda.address.msf.frame = frame % CD_FRAMES_PER_SECOND;
     cdda.nframes = 1;
     cdda.buffer = data;
-#endif
 
-#if  __FreeBSD_version >= 501106
-    if (pread(fd, data, CD_RAW_FRAME_SIZE, frame * CD_RAW_FRAME_SIZE) != CD_RAW_FRAME_SIZE) {
-#else
     /* read a frame */
     if(ioctl(fd, CDIOCREADAUDIO, &cdda) < 0) {
-#endif
       perror("CDIOCREADAUDIO");
       return -1;
     }
@@ -1486,7 +1473,7 @@ static void _cdda_save_cached_cddb_infos(cdda_input_plugin_t *this, char *fileco
     return;
   }
   else {
-    fprintf(fd, "%s", filecontent);
+    fprintf(fd, filecontent);
     fclose(fd);
   }
   
