@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_plugin.c,v 1.37 2004/09/29 18:30:37 miguelfreitas Exp $
+ * $Id: xine_plugin.c,v 1.45 2005/06/17 22:36:43 dsalt Exp $
  *
  * advanced video deinterlacer plugin
  * Jun/2003 by Miguel Freitas
@@ -32,6 +32,7 @@
 #include "xine_internal.h"
 #include "post.h"
 #include "xineutils.h"
+#include "xine_buffer.h"
 #include <pthread.h>
 
 #include "tvtime.h"
@@ -60,6 +61,8 @@ static char *enum_methods[MAX_NUM_METHODS];
 static char *enum_pulldown[] = { "none", "vektor", NULL };
 static char *enum_framerate[] = { "full", "half_top", "half_bottom", NULL };
 
+static void *help_string;
+
 /*
  * this is the struct used by "parameters api" 
  */
@@ -196,7 +199,7 @@ static xine_post_api_descr_t * get_param_descr (void) {
   return &param_descr;
 }
 
-static char * get_help (void) {
+static char * get_static_help (void) {
   return _("Advanced tvtime/deinterlacer plugin with pulldown detection\n"
            "This plugin aims to provide deinterlacing mechanisms comparable "
            "to high quality progressive DVD players and so called "
@@ -244,14 +247,16 @@ static char * get_help (void) {
            "systems to try deinterlace algorithms, in a tradeoff between quality "
            "and cpu usage.\n"
            "\n"
+           "* Uses several algorithms from tvtime and dscaler projects.\n"
            "Deinterlacing methods: (Not all methods are available for all plataforms)\n"
            "\n"
-           "(FIXME: explain each method, check tvtime/dscaler docs... i fell lazy)\n"
-           "\n"
-           "* Uses several algorithms from tvtime and dscaler projects.\n"
            );
 }
 
+static char * get_help (void) {
+  return (char *)help_string;
+}
+
 static xine_post_api_t post_api = {
   set_parameters,
   get_parameters,
@@ -302,24 +307,17 @@ static void *deinterlace_init_plugin(xine_t *xine, void *data)
 
   setup_speedy_calls(xine_mm_accel(),0);
 
-  linear_plugin_init();
-  linearblend_plugin_init();
-  greedy_plugin_init();
-  greedy2frame_plugin_init();
-  weave_plugin_init();
-  double_plugin_init();
-  vfir_plugin_init();
+  register_deinterlace_method( linear_get_method() );
+  register_deinterlace_method( linearblend_get_method() );
+  register_deinterlace_method( greedy_get_method() );
+  register_deinterlace_method( greedy2frame_get_method() );
+  register_deinterlace_method( weave_get_method() );
+  register_deinterlace_method( double_get_method() );
+  register_deinterlace_method( vfir_get_method() );
+  register_deinterlace_method( scalerbob_get_method() );
+  register_deinterlace_method( dscaler_greedyh_get_method() );
+  register_deinterlace_method( dscaler_tomsmocomp_get_method() );
 
-  scalerbob_plugin_init();
-
-  /*
-  dscaler_greedyh_plugin_init();
-  dscaler_twoframe_plugin_init();
-
-  dscaler_videobob_plugin_init();
-  dscaler_videoweave_plugin_init();
-  dscaler_tomsmocomp_plugin_init();
-  */
   filter_deinterlace_methods( config_flags, 5 /*fieldsavailable*/ );
   if( !get_num_deinterlace_methods() ) {
       xprintf(xine, XINE_VERBOSITY_LOG, 
@@ -327,12 +325,37 @@ static void *deinterlace_init_plugin(xine_t *xine, void *data)
       return NULL;
   }
 
+  help_string = xine_buffer_init(1024);
+  xine_buffer_strcat( help_string, get_static_help() );
+
   enum_methods[0] = "use_vo_driver";
   for(i = 0; i < get_num_deinterlace_methods(); i++ ) {
-    enum_methods[i+1] = (char *)get_deinterlace_method(i)->short_name;
+    int j, desc_len;
+    deinterlace_method_t *method;
+
+    method = get_deinterlace_method(i);
+    
+    enum_methods[i+1] = (char *)method->short_name;
+    xine_buffer_strcat( help_string, "[" );
+    xine_buffer_strcat( help_string, (char *)method->short_name );
+    xine_buffer_strcat( help_string, "] " );
+    xine_buffer_strcat( help_string, (char *)method->name );
+    xine_buffer_strcat( help_string, ":\n" );
+
+    desc_len = 0;
+    for(j = 0; j < sizeof(method->description)/sizeof(method->description[0]); j++ ) {
+      if( strlen(method->description[j]) )
+        desc_len = j+1;
+    }
+
+    for(j = 0; j < desc_len; j++ ) {
+      xine_buffer_strcat( help_string, (char *)method->description[j] );
+      xine_buffer_strcat( help_string, "\n" );
+    }
+    xine_buffer_strcat( help_string, "---\n" );
   }
   enum_methods[i+1] = NULL;
-
+  
 
   /* Some default values */
   class->init_param.method                     = 1; /* First (plugin) method available */
@@ -411,6 +434,7 @@ static char *deinterlace_get_description(post_class_t *class_gen)
 
 static void deinterlace_class_dispose(post_class_t *class_gen)
 {
+  xine_buffer_free(help_string);
   free(class_gen);
 }
 
@@ -421,7 +445,6 @@ static void deinterlace_dispose(post_plugin_t *this_gen)
 
   if (_x_post_dispose(this_gen)) {
     _flush_frames(this);
-    pthread_mutex_destroy(&this->lock);
     free(this);
   }
 }
@@ -551,8 +574,7 @@ static int deinterlace_build_output_field(
   int force24fps;
             
   force24fps = this->judder_correction && !this->cheap_mode &&
-               ( (this->pulldown == PULLDOWN_DALIAS) ||
-                 (this->pulldown == PULLDOWN_VEKTOR && this->tvtime->filmmode) );
+               ( this->pulldown == PULLDOWN_VEKTOR && this->tvtime->filmmode );
   
   if( this->tvtime->curmethod->doscalerbob ) {
     scaler = 2;
@@ -563,7 +585,12 @@ static int deinterlace_build_output_field(
     frame->width, frame->height / scaler, frame->ratio, yuy2_frame->format,
     frame->flags | VO_BOTH_FIELDS);
   pthread_mutex_lock (&this->lock);
-    
+
+  deinterlaced_frame->crop_left   = frame->crop_left;
+  deinterlaced_frame->crop_right  = frame->crop_right;
+  deinterlaced_frame->crop_top    = frame->crop_top;
+  deinterlaced_frame->crop_bottom = frame->crop_bottom;
+
   _x_extra_info_merge(deinterlaced_frame->extra_info, frame->extra_info);
     
   if( skip > 0 && !this->pulldown ) {
