@@ -201,8 +201,8 @@ static inline v2g zoomVector(ZoomFilterFXWrapperData *data, float X, float Y)
     /* Noise */
     if (data->noisify)
     {
-        vx += (((float)rand()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
-        vy += (((float)rand()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
+        vx += (((float)random()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
+        vy += (((float)random()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
     }
     
     /* Hypercos */
@@ -684,7 +684,7 @@ static void generatePrecalCoef (int precalCoef[16][16])
 
 /* VisualFX Wrapper */
 
-static void zoomFilterVisualFXWrapper_init (struct _VISUAL_FX *_this)
+static void zoomFilterVisualFXWrapper_init (struct _VISUAL_FX *_this, PluginInfo *info)
 {
     ZoomFilterFXWrapperData *data = (ZoomFilterFXWrapperData*)malloc(sizeof(ZoomFilterFXWrapperData));
     
