@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_xv.c,v 1.209.2.1 2005/04/20 17:21:13 mroi Exp $
+ * $Id: video_out_xv.c,v 1.210 2005/03/06 09:55:35 tmattern Exp $
  *
  * video_out_xv.c, X11 video extension interface for xine
  *
@@ -44,6 +44,12 @@
 #include <errno.h>
 #include <math.h>
 
+#if defined (__SVR4) && defined (__sun)
+# include <sys/int_types.h>
+#else
+# include <stdint.h>
+#endif
+
 #include <sys/types.h>
 #if defined(__FreeBSD__)
 #include <machine/param.h>
