@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -20,7 +20,7 @@
  * Demuxer helper functions
  * hide some xine engine details from demuxers and reduce code duplication
  *
- * $Id: demux.c,v 1.54.2.1 2005/04/20 17:21:15 mroi Exp $ 
+ * $Id: demux.c,v 1.58 2005/07/17 23:11:45 dsalt Exp $ 
  */
 
 
@@ -150,13 +150,13 @@ void _x_demux_control_headers_done (xine_stream_t *stream) {
 
   pthread_mutex_lock (&stream->counter_lock);
 
-  if (stream->video_thread_created) {
+  if (stream->video_thread) {
     header_count_video = stream->header_count_video + 1;
   } else {
     header_count_video = 0;
   }
 
-  if (stream->audio_thread_created) {
+  if (stream->audio_thread) {
     header_count_audio = stream->header_count_audio + 1;
   } else {
     header_count_audio = 0;
@@ -292,9 +292,9 @@ static void *demux_loop (void *stream_gen) {
   lprintf ("loop finished (status: %d)\n", status);
 
   pthread_mutex_lock (&stream->counter_lock);
-  if (stream->audio_thread_created)
+  if (stream->audio_thread)
     finished_count_audio = stream->finished_count_audio + 1;
-  if (stream->video_thread_created)
+  if (stream->video_thread)
     finished_count_video = stream->finished_count_video + 1;
   pthread_mutex_unlock (&stream->counter_lock);
 
@@ -332,13 +332,12 @@ int _x_demux_start_thread (xine_stream_t *stream) {
   
   if( !stream->demux_thread_running ) {
 
-    if (stream->demux_thread_created) {
+    if (stream->demux_thread) {
       void *p;
       pthread_join(stream->demux_thread, &p);
     }
 
     stream->demux_thread_running = 1;
-    stream->demux_thread_created = 1;
     if ((err = pthread_create (&stream->demux_thread,
 			       NULL, demux_loop, (void *)stream)) != 0) {
       printf ("demux: can't create new thread (%s)\n", strerror(err));
@@ -369,9 +368,9 @@ int _x_demux_stop_thread (xine_stream_t *stream) {
 
   lprintf ("joining thread %ld\n", stream->demux_thread );
   
-  if( stream->demux_thread_created ) {
+  if( stream->demux_thread ) {
     pthread_join (stream->demux_thread, &p);
-    stream->demux_thread_created = 0;
+    stream->demux_thread = 0;
   }
   
   /*
@@ -601,3 +600,49 @@ int _x_demux_read_send_data(fifo_buffer_t *fifo, input_plugin_t *input,
   
   return 0;
 }
+
+/*
+ * Helper function for sending MRL reference events
+ */
+void _x_demux_send_mrl_reference (xine_stream_t *stream, int alternative,
+				  const char *mrl, const char *title,
+				  int start_time, int duration)
+{
+  xine_event_t event;
+  union {
+    xine_mrl_reference_data_ext_t *e;
+    xine_mrl_reference_data_t *b;
+  } data;
+  int mrl_len = strlen (mrl);
+
+  if (!title)
+    title = "";
+
+  /* extended MRL reference event */
+
+  event.stream = stream;
+  event.data_length = offsetof (xine_mrl_reference_data_ext_t, mrl) +
+                      mrl_len + strlen (title) + 2;
+  data.e = event.data = malloc (event.data_length);
+    
+  data.e->alternative = alternative;
+  data.e->start_time = start_time;
+  data.e->duration = duration;
+  strcpy (data.e->mrl, mrl);
+  strcpy (data.e->mrl + mrl_len + 1, title ? title : "");
+
+  event.type = XINE_EVENT_MRL_REFERENCE_EXT;
+  xine_event_send (stream, &event);
+
+  /* plain MRL reference event */
+
+  event.data_length = offsetof (xine_mrl_reference_data_t, mrl) + mrl_len + 1;
+
+  /*data.b->alternative = alternative;*/
+  strcpy (data.b->mrl, mrl);
+
+  event.type = XINE_EVENT_MRL_REFERENCE;
+  xine_event_send (stream, &event);
+
+  free (data.e);
+}
