@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: events.c,v 1.26.2.1 2005/07/17 21:50:14 miguelfreitas Exp $
+ * $Id: events.c,v 1.27 2005/06/13 00:32:12 miguelfreitas Exp $
  *
  * Event handling functions
  *
@@ -183,10 +183,6 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
     xine_event_free (event);
   }
 
-  pthread_mutex_destroy(&queue->lock);
-  pthread_cond_destroy(&queue->new_event);
-  pthread_cond_destroy(&queue->events_processed);
-
   free (queue);
 }
 
