@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: load_plugins.c,v 1.193.2.5 2005/07/18 01:46:40 miguelfreitas Exp $
+ * $Id: load_plugins.c,v 1.210 2005/07/18 01:46:02 miguelfreitas Exp $
  *
  *
  * Load input/demux/audio_out/video_out/codec plugins
@@ -32,9 +32,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#ifdef HAVE_DIRENT_H
 #include <dirent.h>
-#endif
 #include <dlfcn.h>
 #include <string.h>
 #include <errno.h>
@@ -538,20 +536,19 @@ static void collect_plugins(xine_t *this, char *path){
     str[path_len + 1] = '\0';
 
     while ((pEntry = readdir (dir)) != NULL) {
-      size_t new_str_size, d_len;
+      size_t new_str_size;
       void *lib = NULL;
       plugin_info_t *info = NULL;
       
       struct stat statbuffer;
 
-      d_len = strlen(pEntry->d_name);
-      new_str_size = path_len + d_len + 2;
+      new_str_size = path_len + pEntry->d_reclen + 2;
       if (str_size < new_str_size) {
 	str_size = new_str_size + new_str_size / 2;
 	str = realloc(str, str_size);
       }
 
-      xine_fast_memcpy(&str[path_len + 1], pEntry->d_name, d_len + 1);
+      xine_fast_memcpy(&str[path_len + 1], pEntry->d_name, pEntry->d_reclen + 1);
 
       if (stat(str, &statbuffer)) {
 	xine_log (this, XINE_LOG_PLUGIN, _("load_plugins: unable to stat %s\n"), str);
@@ -1088,21 +1085,21 @@ void _x_scan_plugins (xine_t *this) {
   if ((pluginpath = getenv("XINE_PLUGIN_PATH")) != NULL) {
     pluginpath = strdup(pluginpath);
   } else {
-    const char *str1, *str2;
+    char *str1, *str2;
     int len;
 
     str1 = "~/.xine/plugins";
     str2 = XINE_PLUGINDIR;
     len = strlen(str1) + strlen(str2) + 2;
     pluginpath = xine_xmalloc(len);
-    snprintf(pluginpath, len, "%s" XINE_DIRECTORY_SEPARATOR_STRING "%s", str1, str2);
+    snprintf(pluginpath, len, "%s:%s", str1, str2);
   }
   plugindir = xine_xmalloc(strlen(pluginpath)+strlen(homedir)+2);
   j=0;
   lenpluginpath = strlen(pluginpath);
   for (i=0; i <= lenpluginpath; ++i){
     switch (pluginpath[i]){
-    case XINE_DIRECTORY_SEPARATOR_CHAR:
+    case ':':
     case '\0':
       plugindir[j] = '\0';
       collect_plugins(this, plugindir);
@@ -2495,9 +2492,7 @@ void _x_dispose_plugins (xine_t *this) {
 
     for (i = 0; this->plugin_catalog->prio_desc[i]; i++)
       free(this->plugin_catalog->prio_desc[i]);
-
-    pthread_mutex_destroy(&this->plugin_catalog->lock);
-
+    
     free (this->plugin_catalog);
   }
 }
