@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.306.2.4 2005/07/21 02:57:52 miguelfreitas Exp $
+ * $Id: xine.c,v 1.314 2005/07/21 02:51:14 miguelfreitas Exp $
  */
 
 /*
@@ -232,7 +232,6 @@ static xine_ticket_t *ticket_init(void) {
   pthread_mutex_init(&port_ticket->lock, NULL);
   pthread_mutex_init(&port_ticket->revoke_lock, NULL);
   pthread_cond_init(&port_ticket->issued, NULL);
-  pthread_cond_init(&port_ticket->revoked, NULL);
   
   return port_ticket;
 }
@@ -289,7 +288,7 @@ static void stop_internal (xine_stream_t *stream) {
   /*
    * stop demux
    */
-  if (stream->demux_plugin && stream->demux_thread_created) {
+  if (stream->demux_plugin && stream->demux_thread) {
     lprintf ("stopping demux\n");
     _x_demux_stop_thread( stream );
     lprintf ("demux stopped\n");
@@ -1342,7 +1341,7 @@ void xine_exit (xine_t *this) {
 #if defined(WIN32)
   WSACleanup();
 #endif
-
+  
   free (this);
 }
 
@@ -1353,7 +1352,7 @@ xine_t *xine_new (void) {
 #ifdef WIN32
     WSADATA Data;
     int i_err;
-#endif
+#endif /*  WIN32 */
 
   this = xine_xmalloc (sizeof (xine_t));
   if (!this)
