@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_interface.c,v 1.87.2.1 2005/04/20 17:21:17 mroi Exp $
+ * $Id: xine_interface.c,v 1.90 2005/07/03 20:31:29 miguelfreitas Exp $
  *
  * convenience/abstraction layer, functions to implement
  * libxine's public interface
@@ -671,6 +671,13 @@ uint32_t xine_get_stream_info (xine_stream_t *stream, int info) {
   case XINE_STREAM_INFO_IGNORE_AUDIO:
   case XINE_STREAM_INFO_IGNORE_SPU:
   case XINE_STREAM_INFO_VIDEO_HAS_STILL:
+  case XINE_STREAM_INFO_VIDEO_AFD:
+  case XINE_STREAM_INFO_DVD_TITLE_NUMBER:
+  case XINE_STREAM_INFO_DVD_TITLE_COUNT:
+  case XINE_STREAM_INFO_DVD_CHAPTER_NUMBER:
+  case XINE_STREAM_INFO_DVD_CHAPTER_COUNT:
+  case XINE_STREAM_INFO_DVD_ANGLE_NUMBER:
+  case XINE_STREAM_INFO_DVD_ANGLE_COUNT:
     return _x_stream_info_get_public(stream, info);
 
   case XINE_STREAM_INFO_MAX_AUDIO_CHANNEL:
